@@ -52,14 +52,14 @@ pub fn entrypoint(args: ArgsOs, stdout: &mut impl Write) -> Result<()> {
     loop {
         match state {
             State::QueryChanged => {
-                output_on_terminal(stdout, &query, &paths[..], selection, columns)?
+                output_on_terminal(stdout, &query, &paths[..], selection, columns, rows)?
             }
             State::PathsChanged => {
-                output_on_terminal(stdout, &query, &paths[..], selection, columns)?;
+                output_on_terminal(stdout, &query, &paths[..], selection, columns, rows)?;
                 state = State::Ready;
             }
             State::SelectionChanged => {
-                output_on_terminal(stdout, &query, &paths[..], selection, columns)?;
+                output_on_terminal(stdout, &query, &paths[..], selection, columns, rows)?;
                 state = State::Ready;
             }
             _ => (),
@@ -67,19 +67,11 @@ pub fn entrypoint(args: ArgsOs, stdout: &mut impl Write) -> Result<()> {
 
         if event::poll(Duration::from_millis(300))? {
             let ev = event::read()?;
-            if ev
-                == Event::Key(KeyEvent {
-                    code: KeyCode::Char('c'),
-                    modifiers: KeyModifiers::CONTROL,
-                })
-                || ev == Event::Key(KeyCode::Esc.into())
-            {
-                // CTRL-C does not send SIGINT even on UNIX/Linux because it's in Raw mode.
-                // Also, we handle Esc as the same.
+            if should_just_exit(&ev) {
                 break;
             } else if let Event::Key(KeyEvent {
                 code: KeyCode::Char(c),
-                modifiers: _,
+                modifiers: KeyModifiers::NONE | KeyModifiers::SHIFT,
             }) = ev
             {
                 query.push(c);
@@ -87,12 +79,12 @@ pub fn entrypoint(args: ArgsOs, stdout: &mut impl Write) -> Result<()> {
             } else if ev == Event::Key(KeyCode::Backspace.into()) {
                 query.pop();
                 state = State::QueryChanged;
-            } else if ev == Event::Key(KeyCode::Up.into()) {
+            } else if should_move_up(&ev) {
                 if selection > 0 {
                     selection -= 1;
                 }
                 state = State::SelectionChanged;
-            } else if ev == Event::Key(KeyCode::Down.into()) {
+            } else if should_move_down(&ev) {
                 if selection < paths_rows(rows) - 1 {
                     selection += 1;
                 }
@@ -104,6 +96,12 @@ pub fn entrypoint(args: ArgsOs, stdout: &mut impl Write) -> Result<()> {
             } else if let Event::Resize(c, r) = ev {
                 columns = c;
                 rows = r;
+                selection = if selection > r {
+                    paths_rows(r) - 1
+                } else {
+                    selection
+                };
+                paths = find_paths(&starting_point, &query, paths_rows(rows))?;
                 state = State::PathsChanged;
             }
         } else if let State::QueryChanged = state {
@@ -132,7 +130,34 @@ enum State<'a> {
 
 fn paths_rows(row: u16) -> u16 {
     // TODO: raise an error when the number of rows is too small.
-    row - 1
+    row - 3
+}
+
+fn should_just_exit(ev: &Event) -> bool {
+    // CTRL-C does not send SIGINT even on UNIX/Linux because it's in Raw mode.
+    // Also, we handle Esc as the same.
+    ev == &Event::Key(KeyEvent {
+        code: KeyCode::Char('c'),
+        modifiers: KeyModifiers::CONTROL,
+    }) || ev == &Event::Key(KeyCode::Esc.into())
+}
+
+fn should_move_up(ev: &Event) -> bool {
+    ev == &Event::Key(KeyCode::Up.into())
+        || ev
+            == &Event::Key(KeyEvent {
+                code: KeyCode::Char('p'),
+                modifiers: KeyModifiers::CONTROL,
+            })
+}
+
+fn should_move_down(ev: &Event) -> bool {
+    ev == &Event::Key(KeyCode::Down.into())
+        || ev
+            == &Event::Key(KeyEvent {
+                code: KeyCode::Char('n'),
+                modifiers: KeyModifiers::CONTROL,
+            })
 }
 
 fn print_and_flush(buffer: &mut impl Write, content: &str) -> io::Result<()> {
@@ -153,6 +178,7 @@ fn output_on_terminal(
     paths: &[MatchedPath],
     selection: u16,
     max_columns: u16,
+    max_rows: u16,
 ) -> Result<()> {
     queue!(
         stdout,
@@ -163,13 +189,20 @@ fn output_on_terminal(
         cursor::SavePosition,
         cursor::MoveToNextLine(1),
     )?;
+
+    let max_path_width: usize = max_columns as usize - 2; // The prefix "> " requires 2 columns.
+    let mut selected: Option<&MatchedPath> = None;
     for (idx, path) in paths.iter().enumerate() {
         let idx = idx as u16;
-        let prefix = if idx == selection { "> " } else { "  " };
+        let prefix = if idx == selection {
+            selected = Some(path);
+            "> "
+        } else {
+            "  "
+        };
         queue!(stdout, style::Print(prefix))?;
 
-        let max_columns: usize = max_columns as usize - 2; // The prefix "> " requires 2 columns.
-        for chunk in path.chunks(max_columns) {
+        for chunk in path.relative_chunks(max_path_width) {
             if chunk.matched() {
                 queue!(
                     stdout,
@@ -183,11 +216,58 @@ fn output_on_terminal(
         }
         queue!(stdout, cursor::MoveToNextLine(1))?;
     }
+
+    queue!(stdout, cursor::MoveToRow(max_rows - 1))?;
+    let selected = &selected
+        .map(|s| s.truncated_absolute(max_columns as usize))
+        .unwrap_or_else(|| "No matching files found.".to_string());
+    status_line(stdout, max_columns as usize, selected)?;
+    help_line(stdout, max_columns as usize)?;
     queue!(stdout, cursor::RestorePosition)?;
     stdout.flush()?;
     Ok(())
 }
 
+fn status_line(stdout: &mut impl Write, max_columns: usize, selected: &str) -> Result<()> {
+    queue!(
+        stdout,
+        style::SetAttribute(Attribute::Bold),
+        style::SetAttribute(Attribute::Reverse),
+        style::Print(format!("{:width$}", selected, width = max_columns)),
+        style::SetAttribute(Attribute::Reset),
+    )?;
+    Ok(())
+}
+
+fn help_line(stdout: &mut impl Write, max_columns: usize) -> Result<()> {
+    // TODO: This number is the same amount of columns occupied by this status line.
+    if max_columns < 58 {
+        return Ok(());
+    }
+    queue!(
+        stdout,
+        cursor::MoveToNextLine(1),
+        style::SetAttribute(Attribute::Bold),
+        style::Print(format!("<Up>/<Ctrl-p>:")),
+        style::SetAttribute(Attribute::Reset),
+        cursor::MoveRight(1),
+        style::Print(format!("Up")),
+        cursor::MoveRight(2),
+        style::SetAttribute(Attribute::Bold),
+        style::Print(format!("<Down>/<Ctrl-n>:")),
+        style::SetAttribute(Attribute::Reset),
+        cursor::MoveRight(1),
+        style::Print(format!("Down")),
+        cursor::MoveRight(2),
+        style::SetAttribute(Attribute::Bold),
+        style::Print(format!("<Enter>:")),
+        style::SetAttribute(Attribute::Reset),
+        cursor::MoveRight(1),
+        style::Print(format!("Execute")),
+    )?;
+    Ok(())
+}
+
 fn find_paths(starting_point: &StartingPoint, query: &str, limit: u16) -> Result<Vec<MatchedPath>> {
     let mut paths = Vec::with_capacity(100); // TODO: Tune this capacity later.
 
